unit PinteraLabels.Product;

interface

uses
	System.SysUtils, System.Classes, System.Types, FMX.Types, FMX.Controls, FMX.Layouts,
	FMX.Objects, FMX.Styles.Objects, FMX.Styles, FMX.GRaphics, FMX.MultiResBitmap, Rtti;

type
	TPinteraProductPanel = class( TStyledControl )
	private
		FProductRectangle : TRectangle;
		FTitle, FDescription : string;
		FImage : TImage;
		FOnClick : TNotifyEvent;

		procedure SetLabelName( const ALabelName : string );
		procedure SetDescription( const ADescription : string );
		function GetBitmap : TBitmap;
		procedure SetBitmap( const ABitmap : TBitmap );
		procedure CreateBitmapIfItDoesntExist;
		procedure OnClickHandler( ASender : TObject );

	protected
		FTitleText, FDescriptionText : TText;
		FBitmap : TBitmap;

		function GetDefaultSize : TSizeF; override;
		function GetStyleObject( const Clone : Boolean ) : TFmxObject; override;
		function GetData : TValue; override;
		procedure SetData( const AData : TValue ); override;

		procedure ApplyStyle; override;
		procedure FreeStyle; override;

	public
		constructor Create( AOwner : TComponent ); override;

	published
		property Title : string read FTitle write SetLabelName;
		property Description : string read FDescription write SetDescription;
		property Bitmap : TBitmap read GetBitmap write SetBitmap;

		property Align;
		property HitTest;
		property Position;
		property OnClick : TNotifyEvent read FOnClick write FOnClick;

	end;

procedure Register;

implementation

const
	LayoutStyle = 'PLProductStyle';
	TitleStyle = 'Title';
	DescriptionStyle = 'Description';
	ImageStyle = 'Image';
	ProductStyle = 'PLProductStyle';

{$R PinteraLabels.Product.res}

procedure Register;
begin
	RegisterComponents( 'EasyLabels', [ TPinteraProductPanel ] );
end;

{ TProduct }

constructor TPinteraProductPanel.Create( AOwner : TComponent );
begin

	inherited;

	FTitle := 'Title';
	FDescription := 'Description';
	FBitmap := TBitmap.Create;

end;

procedure TPinteraProductPanel.CreateBitmapIfItDoesntExist;
begin

	if not Assigned( FBitmap ) then
		FBitmap := TBitmap.Create;

end;

procedure TPinteraProductPanel.ApplyStyle;
begin

	inherited;

	if not Assigned( FProductRectangle ) then
		FProductRectangle := FindStyleResource( 'ProductRectangleStyle' ) as TRectangle;
	if Assigned( FProductRectangle ) then
	begin

		FProductRectangle.OnClick := OnClickHandler;

  end;

	if not Assigned( FTitleText ) then
		FTitleText := FindStyleResource( TitleStyle ) as TText;
	if Assigned( FTitleText ) then
		FTitleText.Text := FTitle;

	if not Assigned( FDescriptionText ) then
		FDescriptionText := FindStyleResource( DescriptionStyle ) as TText;
	if Assigned( FDescriptionText ) then
		FDescriptionText.Text := FDescription;

	if not Assigned( FImage ) then
		FImage := FindStyleResource( ImageStyle ) as TImage;
	if Assigned( FImage ) then
	begin

		if Assigned( FImage.Bitmap ) and Assigned( FBitmap ) then
			FImage.Bitmap.Assign( FBitmap );

	end;

end;

procedure TPinteraProductPanel.FreeStyle;
begin

	inherited;

	FProductRectangle := nil;
	FTitleText := nil;
	FDescriptionText := nil;
	FImage := nil;

end;

function TPinteraProductPanel.GetData : TValue;
begin

	Result := FTitle;

end;

function TPinteraProductPanel.GetDefaultSize : TSizeF;
begin

	Result := TSizeF.Create( 320, 90 );

end;

function TPinteraProductPanel.GetBitmap : TBitmap;
begin

	CreateBitmapIfItDoesntExist;
	Result := FBitmap;

end;

function TPinteraProductPanel.GetStyleObject( const Clone : Boolean ) : TFmxObject;
begin

	if ( StyleLookup = '' ) then
		Result := TFmxObject( TStyleStreaming.LoadFromResource( HInstance, ProductStyle, RT_RCDATA ) )
	else
		Result := inherited GetStyleObject( Clone );

end;

procedure TPinteraProductPanel.OnClickHandler( ASender : TObject );
begin

	if Assigned( FOnClick ) then
		FOnClick( Self );

end;

procedure TPinteraProductPanel.SetData( const AData : TValue );
begin

	if AData.IsType<string> then
		FTitle := AData.AsString
	else if AData.IsOrdinal then
		FTitle := AData.AsOrdinal.ToString
	else
		FTitle := 'N/A';

end;

procedure TPinteraProductPanel.SetDescription( const ADescription : string );
begin

	FDescription := ADescription;

	ApplyStyle;

end;

procedure TPinteraProductPanel.SetBitmap( const ABitmap : TBitmap );
begin

	CreateBitmapIfItDoesntExist;
	FBitmap.Assign( ABitmap );

	ApplyStyle;

end;

procedure TPinteraProductPanel.SetLabelName( const ALabelName : string );
begin

	FTitle := ALabelName;

	ApplyStyle;

end;

end.
